#
# Transfer interface type to target-side code

getsym $coff _iftype
PutLong $DEV $_iftype_addr $_iftype_pg $iftype

#
# Transfer controller base address to target-side code
#
getsym $coff _ctrlr_base
PutLong $DEV $_ctrlr_base_addr $_ctrlr_base_pg $ctrlr_base

#
# CS pins autoplacement
# if user specifies "set cs_port_address auto" 
# then define CS, connected to controller's dedicated pin
#

set autocs 0

if { [string equal -nocase $cs_port_address "auto"] } {

  if { [string equal $cpu_type "TMS470R1B"] } {
    # cs resides in SPIPC3 register
    set cs_port_address [expr $ctrlr_base + 0x24]
    set cs_port_mempage 0
    # register is 32-bit wide
    set cs_port_width 2
    # cs is controlled by bit 4 (0x10)
    set cs_port_bit 0x10
    # cs polarity is active low
    set cs_port_polarity 0
    #
    set autocs 1
  }

  if { [string equal $cpu_type "TMS320C674x"] } {
    # cs resides in SPIPC3 register
    set cs_port_address [expr $ctrlr_base + 0x20]
    set cs_port_mempage 0
    # register is 32-bit wide
    set cs_port_width 2
    # cs is controlled by bit N
    set cs_port_bit [expr 1 << $cs_number]
    # cs polarity is active low
    set cs_port_polarity 0
    #
    set autocs 1
  }

}

#
# Transfer CS pin info to target-side code
#
getsym $coff _cs_port_address
getsym $coff _cs_port_mempage
getsym $coff _cs_port_width
getsym $coff _cs_port_bit
getsym $coff _cs_port_polarity

PutLong $DEV $_cs_port_address_addr $_cs_port_address_pg $cs_port_address
PutLong $DEV $_cs_port_mempage_addr $_cs_port_mempage_pg $cs_port_mempage
PutLong $DEV $_cs_port_width_addr $_cs_port_width_pg $cs_port_width
PutLong $DEV $_cs_port_bit_addr $_cs_port_bit_pg $cs_port_bit
PutLong $DEV $_cs_port_polarity_addr $_cs_port_polarity_pg $cs_port_polarity

#
# Initialize SPI controller
#

if { [string equal $cpu_type "TMS470R1B"] } {

  puts "Initializing SPI controller (@$ctrlr_base)..." 

  #
  # Calculate divide factor for ~ 1 MHz SCK
  #

  set divisor [expr $periph_clock / 1000000]
  puts "Calculated CLKDIV value is $divisor"

  # SPICTRL2 = 0x00000000
  PutLong $DEV [expr $ctrlr_base + 0x04] 0 0x00000000
  # pause
  after 10

  # SPICTRL1; divisor and bit count
  PutLong $DEV [expr $ctrlr_base + 0x00] 0 [expr ($divisor << 5) | 8]

  # SPICTRL2; PHASE=1 CLKMOD=1 MASTER=1
  PutLong $DEV [expr $ctrlr_base + 0x04] 0 0x00000029

  # SPICTRL3; 0
  PutLong $DEV [expr $ctrlr_base + 0x08] 0 0x00000000

  # pins: CS and ENA - as GPIO
  # SPIPC6;
  PutLong $DEV [expr $ctrlr_base + 0x30] 0 0x0000000E

  # directions - CS as output
  # SPIPC1;
  PutLong $DEV [expr $ctrlr_base + 0x1C] 0 0x00000010
  
  # value - CS = 1
  # SPIPC3;
  PutLong $DEV [expr $ctrlr_base + 0x24] 0 0x00000010

  # SPICTRL2; out of reset
  PutLong $DEV [expr $ctrlr_base + 0x04] 0 0x00000039

  # dummy read from SPIBUF to clear overrun and rdy flags
  ReadLong $DEV [expr $ctrlr_base + 0x14] 0

}

if { [string equal $cpu_type "TMS320C2800"] } {

  set mempage 1
  puts "Initializing SPI controller ($ctrlr_base)..." 

  #
  # Calculate divide factor for ~ 1 MHz SCK
  #

  set divisor [expr $periph_clock / 1000000-1]

  puts "Calculated CLKDIV value is $divisor"
  

  if { [string equal $cpu_type "TMS320C5500"] } {

    set mempage 2

  }

  puts "Initializing SPI controller with base adress $ctrlr_base ..." 

  #SPIPRI = 0x10
  PutShort $DEV [expr $ctrlr_base + 15] $mempage 0x10
  puts [format "SPIPRI=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 15] 1]] 

  # SPIFFTX = 0xe040
  PutShort $DEV [expr $ctrlr_base + 10] $mempage 0xe040
  puts [format "SPIFFTX=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 10] 1]] 
  

  #SPIFFRX = 0x204f
  PutShort $DEV [expr $ctrlr_base + 11] $mempage 0x204f
  puts [format "SPIFFRX=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 11] 1]] 

  #SPIFFCT = 0x0
  PutShort $DEV [expr $ctrlr_base + 12] $mempage 0x0000
  puts [format "SPIFFCT=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 12] 1]] 

  #SPICCR = 0x0007
  PutShort $DEV [expr $ctrlr_base + 0]  $mempage 0x07
  puts [format "SPICCR=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 0] 1]] 

  #SPICTL = 0x0006
  PutShort $DEV [expr $ctrlr_base + 1] $mempage 0x06
  puts [format "SPICTL=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 1] 1]] 

  #SPIBRR = divisor
  PutShort $DEV [expr $ctrlr_base + 4] $mempage $divisor
  puts [format "SPIBRR=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 4] 1]] 

  #SPICCR = 0x00c7
  PutShort $DEV [expr $ctrlr_base + 0] $mempage 0xC7
  puts [format "SPICCR=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 0] 1]] 

  puts "Initializing SPI controller Finished"
}


if { [string equal $cpu_type "TMS320C674x"] } {

   set PINMUX3 [expr 0x01C14000 + 0x12C]
   set PINMUX4 [expr 0x01C14000 + 0x130]
   set PINMUX5 [expr 0x01C14000 + 0x134]

   if { $ifnum == 0 } {

     set SPICSMUX [list [list $PINMUX4 1] \
                        [list $PINMUX4 0] \
                        [list $PINMUX3 7] \
                        [list $PINMUX3 6] \
                        [list $PINMUX3 5] \
                        [list $PINMUX3 4] \
                        [list $PINMUX4 1] \
                        [list $PINMUX4 0] ]
   
#    SPI0 (clk, data, NOT CS!)

     set rval [ ReadLong $DEV $PINMUX3 0 ]
     set rval [ expr $rval & ~0x0000FF0F ]
     set rval [ expr $rval |  0x00001101 ]
     PutLong $DEV $PINMUX3 0 $rval

     puts "SPI0 pinmux configured"

   }


   if { $ifnum == 1 } {
   
     set SPICSMUX [list [list $PINMUX5 1] \
                        [list $PINMUX5 0] \
                        [list $PINMUX4 7] \
                        [list $PINMUX4 6] \
                        [list $PINMUX4 5] \
                        [list $PINMUX4 4] \
                        [list $PINMUX4 3] \
                        [list $PINMUX4 2] ]

#    SPI1 (clk, data, NOT CS!)

     set rval [ ReadLong $DEV $PINMUX5 0 ]
     set rval [ expr $rval & ~0x00FF0F00 ]
     set rval [ expr $rval |  0x00110100 ]
     PutLong $DEV $PINMUX5 0 $rval

     puts "SPI1 pinmux configured"

   }

   if { $ifnum == 1 || $ifnum == 0 } {

     set port  [lindex [lindex $SPICSMUX $cs_number] 0]
     set field [lindex [lindex $SPICSMUX $cs_number] 1]

     set rval [ ReadLong $DEV $port 0 ]
     set rval [ expr $rval & ~(0x0F << ($field * 4))]
     set rval [ expr $rval |  (0x01 << ($field * 4))]
     PutLong $DEV $port 0 $rval

     puts [format "Mux for CS: fld %i in port 0x%08X (0x%08X)" $field $port $rval]


   }


   set SPIGCR0 [expr $ctrlr_base + 0x00]
   set SPIGCR1 [expr $ctrlr_base + 0x04]
   set SPIINT0 [expr $ctrlr_base + 0x08]
   set SPILVL [expr $ctrlr_base + 0x0C]
   set SPIFLG [expr $ctrlr_base + 0x10]
   set SPIPC0 [expr $ctrlr_base + 0x14]
   set SPIPC1 [expr $ctrlr_base + 0x18]
   set SPIPC2 [expr $ctrlr_base + 0x1C]
   set SPIPC3 [expr $ctrlr_base + 0x20]
   set SPIPC4 [expr $ctrlr_base + 0x24]
   set SPIPC5 [expr $ctrlr_base + 0x28]
   set SPIDAT0 [expr $ctrlr_base + 0x38]
   set SPIDAT1 [expr $ctrlr_base + 0x3C]
   set SPIBUF [expr $ctrlr_base + 0x40]
   set SPIEMU [expr $ctrlr_base + 0x44]
   set SPIDELAY [expr $ctrlr_base + 0x48]
   set SPIDEF [expr $ctrlr_base + 0x4C]
   set SPIFMT0 [expr $ctrlr_base + 0x50]
   set SPIFMT1 [expr $ctrlr_base + 0x54]
   set SPIFMT2 [expr $ctrlr_base + 0x58]
   set SPIFMT3 [expr $ctrlr_base + 0x5c]
   set INTVEC1 [expr $ctrlr_base + 0x64]

   # Reset
   PutLong $DEV $SPIGCR0 0 0x00000000
   # out of reset
   PutLong $DEV $SPIGCR0 0 0x00000001
   # delay
   ReadLong $DEV $SPIFLG 0
   # Master mode
   PutLong $DEV $SPIGCR1 0 0x00000003
   # CLK, SOMI, SIMO -> func. pins
   PutLong $DEV $SPIPC0 0 0x00000E00
   # Select Data Format 0
   PutShort $DEV [expr $SPIDAT1+2] 0 0x0000
   # Data format 0 (div24, 8bit data, phase=1 polarity=0)
   PutLong $DEV $SPIFMT0 0 0x00010708
   # Master mode + ENABLE
   PutLong $DEV $SPIGCR1 0 0x01000003

   if { $autocs } {

   # Enable CS

     set rval [ ReadLong $DEV $SPIPC1 0 ]
     set rval [ expr $rval & ~(1 << $cs_number) ]
     set rval [ expr $rval |  (1 << $cs_number) ]
     PutLong $DEV $SPIPC1 0 $rval
     puts "Autocs: CS$cs_number pin enabled"

   }

   ReadLong $DSP $SPIFLG 0
   PutLong $DEV $SPIFLG 0 0x15F

   puts "SPI$ifnum configured and enabled"

}
