#include "../src/comdef.h"
#include "../src/spi-eeprom.h"

/*
  Set or Reset CS signal
*/

static void SetCS(uint16 fActive)
{

  uint32 bit;
  uint32 addr = cs_port_address;
  uint32 data;

  #if defined(__TMS320C55X__)
    /* for C55 - adjust address if MEMPAGE 0 (code) */
	/* convert byte-address to word-address */
    if (cs_port_mempage == 0)
	  addr >>= 1; 
  #endif

  bit = fActive ? (cs_port_polarity ? cs_port_bit : 0) : (cs_port_polarity ? 0 : cs_port_bit);

  switch (cs_port_width) {
    case 0:
      #if defined(__TMS320C55X__)
	  if (cs_port_mempage == 2) {
	    data = (*(volatile ioport uint8*)(addr)) & (~cs_port_bit);
        (*(volatile ioport uint8*)(addr)) = data  | bit;
	  } else 
	  #endif
	  {
  	    data = (*(volatile uint8*)(addr)) & (~cs_port_bit);
        (*(volatile uint8*)(addr)) = data | bit;
      }
	  break;
    case 1:
      #if defined(__TMS320C55X__) 
	  if (cs_port_mempage == 2) {
	    data = (*(volatile ioport uint16*)(addr)) & (~cs_port_bit);
        (*(volatile ioport uint16*)(addr)) = data | bit;
	  } else 
	  #endif
	  {
  	    data = (*(volatile uint16*)(addr)) & (~cs_port_bit);
        (*(volatile uint16*)(addr)) = data | bit;
      }
	  break;
    case 2:
      #if defined(__TMS320C55X__)
	  if (cs_port_mempage == 2) {
	    data = (*(volatile ioport uint32*)(addr)) & (~cs_port_bit);
        (*(volatile ioport uint32*)(addr)) = data | bit;
	  } else 
	  #endif
	  {
  	    data = (*(volatile uint32*)(addr)) & (~cs_port_bit);
        (*(volatile uint32*)(addr)) = data | bit;
      }
	  break;
  }
}

static uint16 SPI_tx(uint16 data)
{

  /************************************************/
  /* CPU's with 16-byte access to controller ports*/

  #if defined(__TMS320C55X__) || \
    defined(__TMS320C28XX__)

    volatile ioport uint16 *ptr = (ioport uint16 *)(ctrlr_base);

	/* MCBSP */
    #if defined(__TMS320C55X__) || \
    defined(__TMS320C28XX__)

	if (iftype == 2)
 	 {

      do {} while (!(*(ptr+SPCR2) & 0x0002));

  	  *(ptr + DXR1) = data;

      do {} while (!(*(ptr + SPCR1) & 0x0002));

	  return *(ptr + DRR1) & 0xFF;

    }

	#endif

	/* SPI */
    #if defined(__TMS320C28XX__)
    if (iftype==3) {
      *(ptr + SPITXBUF) = data<<8;		
      do {} while ((*(ptr + SPIFFRX) & 0x1F00)!=0x100);
      return *(ptr + SPIRXBUF); 
    }
    #endif

  /************************************************/
  /* CPU's with 32-byte access to controller ports*/

  #elif defined(_TMS320C6X) || \
        defined(__TMS470__)
    volatile uint32 *ptr = (uint32 *)(ctrlr_base);

	/* MCBSP */
    #if defined(_TMS320C6X)
	if (iftype == 2) /* MCBSP */
	{
      do {} while (!(*(ptr + SPCR) & 0x00020000));

  	  *(ptr + DXR) = data;

      do {} while (!(*(ptr+SPCR) & 0x00000002));

	  return *(ptr + DRR) & 0xFF;
	}
    #endif
	
    if (iftype == 3) /* SPI */
	{
	/* SPI */
    #if defined(__TMS470__)

	  *(ptr + SPIDAT0) = data;

      do {} while (!(*(ptr + SPICTRL3) & 0x00000001));

	  return *(ptr + SPIBUF) & 0xFF;

    #endif

    #if defined(__L138__)
	  uint32 d;

	  *(ptr + (0x38>>2)) = data & 0xFF; // SPIDAT0

      do {} while (!(*(ptr + (0x10>>2)) & 0x00000100)); //SPIFLG

      d = *(ptr + (0x40>>2));   // SPIBUF

	  *(ptr + (0x10>>2)) = 0x100; // SPIFLG

	  return d & 0xFF; // SPIFLG
    #endif
	}

  #endif

  return 0;
}

static void SendWREN()
{
  SetCS(1);
  SPI_tx(0x06);
  SetCS(0);
}

volatile uint16 tstd;

static uint16 wait_for_SPI_EEPROM_ready()
{
  uint16 data;

  /* do RDSR while IC reports "not ready" */
  do {
    SetCS(1);
    SPI_tx(0x05);
    data=SPI_tx(0x00); 
	SetCS(0);
  } while (data & 1);

  return 1;
}

/* WRITE command */
static uint16 send_SPI_dblock(uint16 blksz, uint16 ofs, uint16 stop_flag)
{

#if defined(__TMS320C55X__) || \
    defined(_TMS320C6X) || \
    defined(__TMS320C28XX__) || \
    defined(__TMS470__)

  uint16 addr_bit;

  wait_for_SPI_EEPROM_ready();

  SendWREN();

  /* Assert CS*/
  SetCS(1);

  /* WRITE command */
  switch (c_addr_bytes) {
    case 3:  addr_bit = (address & 0x1000000) >> (16+5);
	         break;
    case 2:  addr_bit = (address & 0x10000) >> (8+5);
	         break;
	default: addr_bit = (address & 0x100) >> (5);
	         break;
  }

  SPI_tx(0x02 | addr_bit);

  /* Send address */
  switch (c_addr_bytes) {
    case 3:  SPI_tx(address >> 16);
    case 2:  SPI_tx(address >> 8);
	default: SPI_tx(address >> 0);
  }

  /* Send data */
  while (blksz) {
    SPI_tx(GetByte((void*)buffer, ofs++));
    --blksz;
  }

  /* Deassert CS */
  SetCS(0);

  return 1;

#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}

static uint16 recv_SPI_dblock(uint16 blksz, uint16 ofs)
{
#if defined(__TMS320C55X__) || \
    defined(_TMS320C6X) || \
    defined(__TMS320C28XX__) || \
    defined(__TMS470__)

  uint16 data;
  uint16 addr_bit;

  wait_for_SPI_EEPROM_ready();

  /* Assert CS*/
  SetCS(1);

  /* READ command */

  switch (c_addr_bytes) {
    case 3:  addr_bit = (address & 0x1000000) >> (16+5);
	         break;
    case 2:  addr_bit = (address & 0x10000) >> (8+5);
	         break;
	default: addr_bit = (address & 0x100) >> (5);
	         break;
  }

  SPI_tx(0x03 | addr_bit);

  /* Send address */
  switch (c_addr_bytes) {
    case 3:  SPI_tx(address >> 16);
    case 2:  SPI_tx(address >> 8);
	default: SPI_tx(address >> 0);
  }

  while (blksz--) {

    data = SPI_tx(0);

	/* put data into receive buffer */
    PutByte((void*)rcvbuf, ofs++, data);

  }
  
  SetCS(0);

  return 1;
#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}


uint16 perform_SPI_eeprom_program()
{
#if defined(__TMS320C55X__) || \
    defined(_TMS320C6X) || \
    defined(__TMS320C28XX__) || \
    defined(__TMS470__)

  uint16 blksz;
  uint16 ofs = 0;
  uint16 i;

  uint16 adj = address % pagelen;
  
  while (length) {
    /* Calculate size of block as minimum of data length in buffer and IC's page length */
    blksz = min(pagelen, length);
    /* if start address not at page boundary, adjust first block length to meet IC's page*/
    blksz -= adj; adj=0;

	/* write block into EEPROM */
	if (!send_SPI_dblock(blksz, ofs, 1))
	  return 0;

    /* Verify data written */
    if (!recv_SPI_dblock(blksz, 0))
      return 0;
    
    for (i=0; i<blksz; i++)
	  if (GetByte((void*)buffer, ofs+i) != GetByte((void*)rcvbuf,i)) {
        status = STATUS_VERIFY_ERR;
		return 0;
	  }

	/* adjust variables */
	ofs += blksz; /* offset in buffer */
    address += blksz; /* address in EEPROM */
	length -= blksz; /* length of data in buffer */
  }

  return 1;
    
#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif

}
 
uint16 perform_SPI_eeprom_erase()
{
#if defined(__TMS320C55X__) || \
    defined(_TMS320C6X) || \
    defined(__TMS320C28XX__) || \
    defined(__TMS470__)

  if (pagelen == sectorlen) {

    uint16 i;
    uint16 pagewords = pagelen >> 1;
    uint16 *ptr = (uint16*)buffer;

    // fill buffer with "pagelen" 0xFF's
	// Erase by programming FF's

    for (i=0; i<pagewords; i++)
      *ptr++ = 0xFFFF;

    // adjust address to begin of page
    address = (address / pagelen) * pagelen;

    // set length to pagelen

    length = pagelen;

    return perform_SPI_eeprom_program();

  } else {
    // Use "Sector Erase" command
    uint16 addr_bit;

    wait_for_SPI_EEPROM_ready();

    SendWREN();

    /* Assert CS*/
    SetCS(1);

    /* SECTOR ERASE command */
    switch (c_addr_bytes) {
      case 3:  addr_bit = (address & 0x1000000) >> (16+5);
  	           break;
      case 2:  addr_bit = (address & 0x10000) >> (8+5);
	           break;
	  default: addr_bit = (address & 0x100) >> (5);
	           break;
    }

    SPI_tx((flash_subtype == 1) ? 0xD8 : (0x52 | addr_bit));

    /* Send address */
    switch (c_addr_bytes) {
      case 3:  SPI_tx(address >> 16);
      case 2:  SPI_tx(address >> 8);
  	  default: SPI_tx(address >> 0);
    }

    /* Deassert CS */
    SetCS(0);

    /* Wait for ERASE completed */
    wait_for_SPI_EEPROM_ready();

    return 1;

  }

#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}
